import fnmatch
import os
from lxml import etree
from .snapshot_release import snapshot_release_publication

SNAPSHOT_TAG_SUFFIX = "SNAPSHOT"


def maven_get_version():
    # read current version
    pom_path = os.path.join(os.environ.get('GITHUB_WORKSPACE'), 'pom.xml')
    pom_doc = etree.parse(pom_path)
    r = pom_doc.xpath('/pom:project/pom:version',
                      namespaces={'pom': 'http://maven.apache.org/POM/4.0.0'})
    return r[0].text


def maven_upload_assets(repo_name, tag_name, release):
    """
          Upload packages produced by maven

    """
    # upload assets
    assets = ['*-bin.tar.gz', '*-bin.zip', '*.jar']
    for dirname, subdirs, files in os.walk(os.environ.get('GITHUB_WORKSPACE')):
        if dirname.endswith('target'):
            for extension in assets:
                for filename in fnmatch.filter(files, extension):
                    with open(os.path.join(dirname, filename), 'rb') as f_asset:
                        release.upload_asset('application/tar+gzip',
                                             filename,
                                             f_asset)


def main():
    snapshot_release_publication(SNAPSHOT_TAG_SUFFIX, maven_get_version, maven_upload_assets)


if __name__ == "__main__":
    main()



import os
import re
import glob
import logging
import glob
from .snapshot_release import snapshot_release_publication

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

SNAPSHOT_TAG_SUFFIX = "+dev"

def python_get_version():
    version = python_get_version_from_init()
    if not version:
        version = python_get_version_from_setup()
    return version


def python_get_version_from_setup():
    setup_path = os.path.join(os.environ.get('GITHUB_WORKSPACE'), 'setup.py')
    prog = re.compile("version=.*")
    try:
        with open(setup_path, 'r') as f:
            for line in f:
                line = line.strip()
                if prog.match(line):
                    version = line[9:-2]
                    logger.info("version {version}")
                    return version
        return None
    except FileNotFoundError:
        return None

def python_get_version_from_init():
    init_path_pattern =  os.path.join(os.environ.get('GITHUB_WORKSPACE'), "*", "__init__.py")
    init_path = glob.glob(init_path_pattern)[0]
    try:
        with open(init_path) as fi:
            result = re.search(r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fi.read())
            if result:
                return result.group(1)
            else:
                return None
    except FileNotFoundError:
        return None


def python_upload_assets(repo_name, tag_name, release):
    """
          Upload packages produced by python setup.py

    """
    package_pattern = os.path.join(os.environ.get('GITHUB_WORKSPACE'),
                                        'dist',
                                        '*')
    packages = glob.glob(package_pattern)
    for package in packages:
        with open(package, 'rb') as f_asset:
            asset_filename = os.path.basename(package)
            logger.info(f"Upload asset file {asset_filename}")
            release.upload_asset('application/zip',
                                 asset_filename,
                                 f_asset)


def main():
    snapshot_release_publication(SNAPSHOT_TAG_SUFFIX, python_get_version, python_upload_assets)


if __name__ == "__main__":
    main()



import unittest
import os
import logging
from pds_github_util.python_snapshot_release import python_get_version

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class PythonGetVersionTestCase(unittest.TestCase):

    def setUp(self):
        current_dir = os.path.dirname(os.path.abspath(__file__))
        os.environ['GITHUB_WORKSPACE'] = os.path.join(current_dir, '..', '..')

    def test_python_get_version(self):
        version = python_get_version()
        logger.info(f"found version is {version}")
        self.assertGreaterEqual(len(version), 3)

    def tearDown(self):
        del os.environ['GITHUB_WORKSPACE']


if __name__ == '__main__':
    unittest.main()



import re
import setuptools

with open("./pds_github_util/__init__.py") as fi:
    result = re.search(r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fi.read())
version = result.group(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pds_github_util", # Replace with your own package name
    version=version,
    license="apache-2.0",
    author="thomas loubrieu",
    author_email="loubrieu@jpl.nasa.gov",
    description="util functions for software life cycle enforcement on github",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NASA-PDS-Incubator/pds-github-util",
    download_url = "https://github.com/NASA-PDS-Incubator/pds-github-util/releases/download/0.0.1/pds_github_util-0.0.1.tar.gz",
    packages=setuptools.find_packages(),
    keywords=['github', 'action', 'github action', 'snapshot', 'release', 'maven'],

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'github3.py',
        'lxml'
    ],
    entry_points={
        # snapshot-release for backward compatibility
        'console_scripts': ['snapshot-release=pds_github_util.maven_snapshot_release:main',
                            'maven-snapshot-release=pds_github_util.maven_snapshot_release:main',
                            'python-snapshot-release=pds_github_util.python_snapshot_release:main'
                            ],
    },


)


# Copyright 2019, California Institute of Technology ("Caltech").
# U.S. Government sponsorship acknowledged.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# * Redistributions must reproduce the above copyright notice, this list of
# conditions and the following disclaimer in the documentation and/or other
# materials provided with the distribution.
# * Neither the name of Caltech nor its operating division, the Jet Propulsion
# Laboratory, nor the names of its contributors may be used to endorse or
# promote products derived from this software without specific prior written
# permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import logging
import github3
import argparse


logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


def delete_snapshot_releases(_repo, suffix):
    """
        Delete all pre-existing snapshot releases
    """
    logger.info("delete previous releases")
    for release in _repo.releases():
        if release.tag_name.endswith(suffix):
            release.delete()


def create_snapshot_release(repo, repo_name, branch_name, tag_name, tagger, upload_assets):
    """
    Create a tag and new release from the latest commit on branch_name.
    Push the assets created in target directory.
    """
    logger.info("create new snapshot release")
    our_branch = repo.branch(branch_name)
    repo.create_tag(tag_name,
                    f'SNAPSHOT release',
                    our_branch.commit.sha,
                    "commit",
                    tagger)

    # create the release
    release = repo.create_release(tag_name, target_commitish=branch_name, name=repo_name + " " + tag_name, prerelease=True)

    logger.info("upload assets")
    upload_assets(repo_name, tag_name, release)


def snapshot_release_publication(suffix, get_version, upload_assets):
    """
    Script made to work in the context of a github action.
    """
    parser = argparse.ArgumentParser(description='Create new snapshot release')
    parser.add_argument('--token', dest='token',
                        help='github personal access token')
    args = parser.parse_args()

    # read organization and repository name
    repo_full_name = os.environ.get('GITHUB_REPOSITORY')
    repo_full_name_array = repo_full_name.split("/")
    org = repo_full_name_array[0]
    repo_name = repo_full_name_array[1]

    tag_name = get_version()
    if tag_name.endswith(suffix):
        tagger = {"name": "thomas loubrieu",
                  "email": "loubrieu@jpl.nasa.gov"}

        gh = github3.login(token=args.token)
        repo = gh.repository(org, repo_name)

        delete_snapshot_releases(repo, suffix)
        create_snapshot_release(repo, repo_name, "master", tag_name, tagger, upload_assets)